@@ -4,12 +4,13 @@ import win32com.client
 import pythoncom
 import time
 import json
+import importlib.resources
 
 from pathlib import Path
-import importlib.resources
 from typing import Union, Any
 from loguru import logger
 from typing import Callable, Optional
+from datetime import datetime, timedelta
 
 from .versions import PlantsimVersion
 from .licenses import PlantsimLicense
@@ -48,6 +49,8 @@ class Plantsim:
         Handler for Plant Simulation events.
     _event_polling_interval : float
         Interval for polling events.
+    _datetime_format : str
+        Format for datetime strings
     _model_loaded : bool
         Whether a model has been loaded.
     _model_path : str
@@ -81,6 +84,7 @@ class Plantsim:
     _event_thread = None
     _event_handler: PlantSimEvents = None
     _event_polling_interval: float = 0.05
+    _datetime_format: str
 
     # State management
     _model_loaded: bool = False
@@ -245,6 +249,8 @@ class Plantsim:
         if self._instance:
             self.quit()
 
+        pythoncom.CoUninitialize()
+
     def set_network(
         self,
         path: PlantsimPath,
@@ -256,7 +262,7 @@ class Plantsim:
         self._instance.SetPathContext(str(self._network_path))
 
         if install_error_handler:
-            self.install_error_handler(path)
+            self.install_error_handler()
 
         if set_event_controller:
             self.set_event_controller()
@@ -491,6 +497,8 @@ class Plantsim:
         except Exception as e:
             raise PlantsimException(e)
 
+        self._set_datetime_format()
+
         self._model_loaded = True
         self._model_path = filepath
         self._simulation_error = None
@@ -501,15 +509,31 @@ class Plantsim:
         resource = f"sim_talk_scripts/{script_name}.st"
         return importlib.resources.files(package).joinpath(resource).read_text()
 
-    def install_error_handler(self, model_path: PlantsimPath):
-        """Installs the Error handler in the model"""
+    def install_error_handler(self):
+        """Installs an error handler in the model file under basis.ErrorHandler. Searches for any method object and duplicates that."""
         simtalk = self._load_simtalk_script("install_error_handler")
 
-        response = self.execute_sim_talk(simtalk, model_path)
+        response = self.execute_sim_talk(simtalk)
 
         if not response:
             raise Exception("Could not create Error Handler")
 
+        self._error_handler = "basis.ErrorHandler"
+
+    def remove_error_handler(self):
+        """Removes the installed error handler from basis.ErrorHandler."""
+        if not self._error_handler:
+            raise Exception("Not error handler has been installed")
+
+        simtalk = self._load_simtalk_script("remove_error_handler")
+
+        response = self.execute_sim_talk(simtalk)
+
+        if not response:
+            raise Exception("Could not remove the error handler")
+
+        self._error_handler = None
+
     def new_model(self, close_other: bool = False) -> None:
         """Creates a new simulation model in the current instance"""
         if close_other:
@@ -598,6 +622,7 @@ class Plantsim:
         on_simulation_error: Optional[
             Callable[["Plantsim", SimulationException], None]
         ] = None,
+        on_progress: Optional[Callable[["Plantsim", float], None]] = None,
     ) -> None:
         """
         Makes a full simulation run and returns after the run is over. Throws in case the simulation ends without finishing.
@@ -607,6 +632,8 @@ class Plantsim:
 
         self.start_simulation(without_animation)
 
+        self._run_simulation_event_loop(on_progress)
+
         while (
             not self._simulation_finished_event.is_set()
             and not self._simulation_error_event.is_set()
@@ -623,7 +650,77 @@ class Plantsim:
         if on_endsim:
             on_endsim(self)
 
-    def stop_simulation(self, eventcontroller_object: str = None) -> None:
+    def _run_simulation_event_loop(
+        self, on_progress: Optional[Callable[["Plantsim", float], None]] = None
+    ):
+        """"""
+        start_date = self.get_start_date()
+        end_time = self.get_end_time()
+        last_progress_update = time.time()
+
+        while (
+            not self._simulation_finished_event.is_set()
+            and not self._simulation_error_event.is_set()
+        ):
+            pythoncom.PumpWaitingMessages()
+            time.sleep(self._event_polling_interval)
+
+            if on_progress:
+                now = time.time()
+                if now - last_progress_update >= 1:
+                    last_progress_update = now
+                    current_simulation_time = self.get_abs_sim_time()
+                    progress = ((current_simulation_time - start_date) / end_time) * 100
+                    on_progress(self, progress)
+
+    def get_abs_sim_time(self) -> datetime:
+        """Gets the current simulation time"""
+        if not self._event_controller:
+            raise Exception("EventController needs to be set.")
+
+        return self._str_to_datetime(
+            self.get_value(PlantsimPath(self._event_controller, "AbsSimTime"))
+        )
+
+    def _str_to_datetime(self, date_str: str) -> datetime:
+        """Converts a string into a datetime"""
+        return datetime.strptime(date_str, self._datetime_format)
+
+    def get_start_date(self) -> datetime:
+        """Extracts the start date from the event controller."""
+        if not self._event_controller:
+            raise Exception("EventController needs to be set.")
+
+        return self._str_to_datetime(
+            self.get_value(PlantsimPath(self._event_controller, "StartDate"))
+        )
+
+    def get_model_language(self) -> int:
+        """Returns the model language."""
+        return self.execute_sim_talk("-> integer return language()")
+
+    def _set_datetime_format(self) -> None:
+        """Sets the datetime format based on the the loaded model."""
+        language = self.get_model_language()
+
+        match language:
+            case 0:  # German
+                self._datetime_format = "%d.%m.%Y %H:%M:%S.%f"
+            case 1:  # English
+                self._datetime_format = "%Y-%m-%d %H:%M:%S.%f"
+            case _:
+                raise NotImplementedError()
+
+    def get_end_time(self) -> timedelta:
+        """Extracts the end time of the event controller."""
+        if not self._event_controller:
+            raise Exception("EventController needs to be set.")
+
+        return timedelta(
+            seconds=self.get_value(PlantsimPath(self._event_controller, "EndTime"))
+        )
+
+    def stop_simulation(self) -> None:
         """
         Stops the simulation
 
@@ -632,7 +729,22 @@ class Plantsim:
         eventcontroller_object : str, optional
             path to the Event Controller object to be reset. If not given, it defaults to the default event controller path (default: None)
         """
-        self._instance.StopSimulation(eventcontroller_object)
+        if not self._event_controller:
+            raise Exception("EventController needs to be set.")
+
+        self._instance.StopSimulation(self._event_controller)
+
+    def set_seed(self, seed: int) -> None:
+        """Sets the Seed on the event controller."""
+        if not self._event_controller:
+            raise Exception("EventController needs to be set")
+
+        if seed > 2147483647 or seed < -2147483647:
+            raise Exception("Seed must be between -2147483647 and 2147483647")
+
+        self.set_value(
+            PlantsimPath(self._event_controller, "RandomNumbersVariant"), seed
+        )
 
     @property
     def simulation_running(self) -> bool:
